#!/bin/bash

[ "$(whoami)" = "root" ] && SUDO="sudo -u wms" || SUDO=""

function log () {
        logger -i -t wms_msmtp $1
}

# make a check before starting envelope a letter
res=$(df -l 2>/dev/null |awk '/\/dev\/shm$/{printf($4)}')
if [ $res -le 10240 ]; then

	# write to syslog
	log "No space left for mails"
	exit 1
fi

# get company id
if [ -z "$1" -a -z "$com_id" ]; then
	com_id=0
elif [ -n "$1" ]; then
	com_id=$1
fi

TMPDIR=/tmp/mail
QUEUEDIR=/var/spool/mail/$com_id

# Set secure permissions on created directories and files
umask 002

# Change to temp directory (create it if necessary)
if [ ! -d "$TMPDIR" ]; then
	$SUDO mkdir -p "$TMPDIR" || exit 1
fi
sudo chown wms.wms -R $TMPDIR
sudo chmod 775 $TMPDIR
cd "$TMPDIR" || exit 1

# Create new unique filenames of the form
# MAILFILE:  ccyy-mm-dd-hh.mm.ss[-x].mail
# MSMTPFILE: ccyy-mm-dd-hh.mm.ss[-x].msmtp
# where x is a consecutive number only appended if you send more than one
# mail per second.
BASE="`date +%Y-%m-%d-%H.%M.%S`-$RANDOM"

if [ -f "$BASE.mail" -o -f "$BASE.msmtp" ]; then
	TMP="$BASE"
	i=1
	while [ -f "$TMP-$i.mail" -o -f "$TMP-$i.msmtp" ]; do
		i=`expr $i + 1`
	done
	BASE="$BASE-$i"
fi
MAILFILE="$BASE.mail"
MSMTPFILE="$BASE.msmtp"

cat | $SUDO tee "$MAILFILE" >/dev/null || exit 1

# Write command line to $MSMTPFILE
# Write the mail to $MAILFILE

# check attach in mails to get cases
# - voicemail, recording - wav
# - faxes - tiff
# - feedback, text - non the both

if [ `grep "Content-Type.*image/tiff" "$MAILFILE" -i -c` -ne 0 ]; then
	# tiff, faxing
	P="fax"
elif [ `grep "Content-Type.*audio/x-wav" "$MAILFILE" -i -c` -ne 0 ]; then
	# recording or voi44cemail
	P="wav"
else
	# just a text
	P="txt"
fi

if [ $(echo "$@" | grep '\-\-' -c) -eq 0 ]; then
	# get mail form mail source
    sed -ne "s/To.*<\(.*\)>/ -- \1/p" "$MAILFILE" | $SUDO tee "$MSMTPFILE" > /dev/null

	# correcting From line
	if [ -f "/etc/companies.d/$com_id/msmtprc" ]; then
		fromline=$(sed -ne 's/from \([@]*\)/\1/p' /etc/companies.d/${com_id}/msmtprc);
		[ -n "$fromline" ] && \
			sed "s/\(From:.*<\)\(pbxservice.*\)>/\1$fromline>/" $MAILFILE | $SUDO tee $MAILFILE >/dev/null;
	fi
	NBASE=$(sed -ne 's/@/[at]/g;s/\s/~/g;s/.*--\~\(.*\)/\1/p;' "$MSMTPFILE")

else
	# mutt version: msmtp-enqueue.sh 0 -- email@domain.com
    echo "$@" | awk '$1 != "--" { $1 = "";} {print $0}' > "$MSMTPFILE"
	NBASE=$(echo "$@" | sed -ne 's/@/[at]/g;s/\s/~/g;s/.*--\~\(.*\)/\1/p;')
fi

$SUDO mv {${BASE},${P}_${NBASE}_${BASE}}.msmtp
$SUDO mv {${BASE},${P}_${NBASE}_${BASE}}.mail

sudo chown wms.wms *msmtp
sudo chown wms.wms *mail

# Move to queue directory (create it if necessary)
if [ ! -d "$QUEUEDIR" ]; then
	$SUDO mkdir -p "$QUEUEDIR" || exit 1
fi
sudo chown wms.wms -R $QUEUEDIR
sudo chmod 775 $QUEUEDIR

$SUDO mv * $QUEUEDIR

exit 0
