local basedir = "/var/www"
local default_cache_path="/var/www/cache"
local realCacheDir = '/tmp/session/lighty/cache'

local helper = require ("common.helper")
local fsutils = require ("core.fsutils")
local bundleLogger = require"core.logger":init('bundle', 1)
local gzip = require ("gzip")

bundleLogger:info('Start')

function isSupportGzip()
    if os.getenv('HTTP_ACCEPT_ENCODING') and os.getenv('HTTP_ACCEPT_ENCODING'):lower():find('gzip') ~= nil then
        return true
    end
    return false
end


function parseRequest()
    local request_uri=os.getenv('REQUEST_URI')

    if request_uri~=nil then
        dir, file, scripts, version = request_uri:match("^(/[^/]+)/cache(/[^?]+)\?([^&]+)&version=(.+)$")
        file =  dir:gsub("collaboration", "cticonnect") .. file
    end

    local t = helper.split(file, "/")

    local fileName = t[table.getn(t)]
    table.remove(t, table.getn(t))
    local type = t[table.getn(t)]
    local cacheDir = default_cache_path..'/'..table.concat(t, '/')
    table.remove(t, table.getn(t))
    local scriptsRootDir = basedir..'/'..table.concat(t, '/')

    if fileName:find('gzip') == nil then
        fileName = 'gzip_'..fileName
    end

    return type, scriptsRootDir, fileName, cacheDir, scripts, version
end


--- Wrapper for reading a full file into a string
-- @param filename Full path to the file
-- @return a string with the content of the file
function readFile(filename)
   local content = ""
   if helper.is_exists(filename) then
      local file = io.open(filename, "r")
      content = file:read("*a")
      io.close(file)
   end
   return content
end


if fsutils.is_link(realCacheDir) == false and fsutils.is_dir(realCacheDir) == false and fsutils.is_file(realCacheDir) == false then
    bundleLogger:info('no dir: '..realCacheDir)
    fsutils.mkdir(realCacheDir);
end

if fsutils.is_link(basedir..'/cache') == false then
    bundleLogger:info('Remove dir: '..default_cache_path)
    fsutils.rmdir(default_cache_path);
end

if fsutils.is_link(basedir..'/cache') == false then
    bundleLogger:info('no link: '..basedir..'/cache')
    os.execute('ln -s '..realCacheDir..' '..basedir..'/cache')
end


local type, scriptsRootDir, fileName, cacheDir, scripts, version = parseRequest()
local cacheFilePath = cacheDir..'/'..fileName

if fsutils.is_dir(cacheDir) == false then
    bundleLogger:info('no dir: '..cacheDir)
    if not fsutils.mkdirr(cacheDir) then
        bundleLogger:error('Error mkdirr: '..cacheDir)
    end
end


local content = ''

if fsutils.is_file(cacheFilePath) == false then
    bundleLogger:info('File not found: '..cacheFilePath)

    local gzFile = gzip.open(cacheFilePath, "wb5")
    if gzFile then
        bundleLogger:info('Write file: '..cacheFilePath)
        if type == 'tpl' then
            local json = require "dkjson";
            gzFile:write("Templates = {\n")
            for _, file in ipairs(helper.split(scripts, ",")) do
                local templateName = file:gsub("/templates/", ".")

                local fileContent = json.encode(readFile(scriptsRootDir.."/modules/"..file..".html"));
                gzFile:write("/*  " .. file .. " */\n\"" .. templateName .. "\": "..fileContent..",\n")
            end
            gzFile:write("}\n")
            gzFile:close()
            -- fsutils.rmfile(cacheFilePath)
        else
            for _, file in ipairs(helper.split(scripts, ",")) do
                file = file:gsub("%.%./", '')
                if file:match(".*\.js$") or file:match(".*\.css$") then
                    gzFile:write("/*  " .. scriptsRootDir..'/' .. file .. " */\n"..readFile(scriptsRootDir..'/' .. file).."\n\n")
                end
            end
            gzFile:close()
        end
    else
        bundleLogger:error('Error open file: '..cacheFilePath)
    end
end


if fsutils.is_file(cacheFilePath) == true then
    bundleLogger:info('File exist: '..cacheFilePath)
    if isSupportGzip() then
        bundleLogger:info('Send compressed data')
        content = readFile(cacheFilePath)
    else
        bundleLogger:info('Send uncompressed data')
        local gzFile = gzip.open(cacheFilePath)
        if gzFile then
            content = gzFile:read("*a")
            gzFile:close()
        end
    end
end


if type == 'js' or type == 'tpl' then
    print("Content-type: application/javascript");
elseif type == 'css' then
    print("Content-type: text/css");
end

print ("Content-length: "..string.len(content).."\n");
io.write(content);

